@@ -45,6 +45,17 @@ static void             cb_row_activated                       (XfmpcDbbrowser *
                                                                 GtkTreeViewColumn *column);
 static gboolean         cb_key_pressed                         (XfmpcDbbrowser *dbbrowser,
                                                                 GdkEventKey *event);
+static gboolean         cb_button_pressed                      (XfmpcDbbrowser *dbbrowser,
+                                                                GdkEventButton *event);
+static gboolean         cb_popup_menu                          (XfmpcDbbrowser *dbbrowser);
+static void             popup_menu                             (XfmpcDbbrowser *dbbrowser);
+
+static void             cb_search_entry_activated              (XfmpcDbbrowser *dbbrowser);
+static gboolean         cb_search_entry_key_released           (XfmpcDbbrowser *dbbrowser,
+                                                                GdkEventKey *event);
+static void             cb_search_entry_changed                (XfmpcDbbrowser *dbbrowser);
+static gboolean         timeout_search                         (XfmpcDbbrowser *dbbrowser);
+static void             timeout_search_destroy                 (XfmpcDbbrowser *dbbrowser);
 
 
 
@@ -79,6 +90,10 @@ struct _XfmpcDbbrowserPrivate
   GtkWidget                *treeview;
   GtkListStore             *store;
   GtkWidget                *search_entry;
+  GtkWidget                *menu;
+
+  guint                     search_timeout;
+  gboolean                  is_searching;
 
   gchar                    *wdir;
   gchar                    *last_wdir;
@@ -180,28 +195,51 @@ xfmpc_dbbrowser_init (XfmpcDbbrowser *dbbrowser)
                                                "text", COLUMN_BASENAME,
                                                NULL);
 
-  /* === Scrolled window === */
+  /* Scrolled window */
   GtkWidget *scrolled = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_ALWAYS);
 
+  /* Menu */
+  priv->menu = gtk_menu_new ();
+
+  GtkWidget *mi = gtk_image_menu_item_new_from_stock (GTK_STOCK_ADD, NULL);
+  gtk_menu_shell_append (GTK_MENU_SHELL (priv->menu), mi);
+  g_signal_connect_swapped (mi, "activate",
+                            G_CALLBACK (xfmpc_dbbrowser_add_selected_rows), dbbrowser);
+
+  gtk_widget_show_all (priv->menu);
+
+  /* === Search entry === */
+  priv->search_entry = gtk_entry_new ();
+
   /* === Containers === */
   gtk_container_add (GTK_CONTAINER (scrolled), priv->treeview);
-  gtk_container_add (GTK_CONTAINER (dbbrowser), scrolled);
+  gtk_box_pack_start (GTK_BOX (dbbrowser), scrolled, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (dbbrowser), priv->search_entry, FALSE, FALSE, 0);
 
   /* === Signals === */
   g_signal_connect_swapped (dbbrowser->mpdclient, "connected",
                             G_CALLBACK (xfmpc_dbbrowser_reload), dbbrowser);
-#if 0
   g_signal_connect_swapped (dbbrowser->mpdclient, "database-changed",
-                            G_CALLBACK (cb_database_changed), dbbrowser);
-#endif
+                            G_CALLBACK (xfmpc_dbbrowser_reload), dbbrowser);
   /* Tree view */
   g_signal_connect_swapped (priv->treeview, "row-activated",
                             G_CALLBACK (cb_row_activated), dbbrowser);
   g_signal_connect_swapped (priv->treeview, "key-press-event",
                             G_CALLBACK (cb_key_pressed), dbbrowser);
+  g_signal_connect_swapped (priv->treeview, "button-press-event",
+                            G_CALLBACK (cb_button_pressed), dbbrowser);
+  g_signal_connect_swapped (priv->treeview, "popup-menu",
+                            G_CALLBACK (cb_popup_menu), dbbrowser);
+  /* Search entry */
+  g_signal_connect_swapped (priv->search_entry, "activate",
+                            G_CALLBACK (cb_search_entry_activated), dbbrowser);
+  g_signal_connect_swapped (priv->search_entry, "key-release-event",
+                            G_CALLBACK (cb_search_entry_key_released), dbbrowser);
+  g_signal_connect_swapped (priv->search_entry, "changed",
+                            G_CALLBACK (cb_search_entry_changed), dbbrowser);
 }
 
 static void
@@ -304,6 +342,11 @@ xfmpc_dbbrowser_reload (XfmpcDbbrowser *dbbrowser)
   gboolean                  is_dir;
   gint                      i = 0;
 
+  if (G_UNLIKELY (!xfmpc_mpdclient_is_connected (dbbrowser->mpdclient)))
+    return;
+  else if (priv->is_searching)
+    return;
+
   xfmpc_dbbrowser_clear (dbbrowser);
 
   if (!xfmpc_dbbrowser_wdir_is_root (dbbrowser))
@@ -338,6 +381,64 @@ xfmpc_dbbrowser_reload (XfmpcDbbrowser *dbbrowser)
     }
 }
 
+void
+xfmpc_dbbrowser_search (XfmpcDbbrowser *dbbrowser,
+                        const gchar *query)
+{
+  XfmpcDbbrowserPrivate    *priv = XFMPC_DBBROWSER_GET_PRIVATE (dbbrowser);
+  gchar                    *filename;
+  gchar                    *basename;
+  gint                      i = 0;
+  static gboolean           no_result, no_result_buf;
+  GdkColor                  color = {0, 0xFFFF, 0x6666, 0x6666};
+
+  if (G_UNLIKELY (!xfmpc_mpdclient_is_connected (dbbrowser->mpdclient)))
+    return;
+
+  priv->is_searching = TRUE;
+  xfmpc_dbbrowser_clear (dbbrowser);
+
+  while (xfmpc_mpdclient_database_search (dbbrowser->mpdclient, query, &filename, &basename))
+    {
+      xfmpc_dbbrowser_append (dbbrowser, filename, basename, FALSE);
+      g_free (filename);
+      g_free (basename);
+      i++;
+    }
+
+  /* check the rightness of the query and then play with the colors */
+  no_result_buf = no_result;
+  if (i == 0)
+    no_result = TRUE;
+  else if (no_result)
+    no_result = FALSE;
+
+  if (no_result != no_result_buf)
+    {
+#ifdef MORE_FUNKY_COLOR_ON_SEARCH_ENTRY
+      gtk_widget_modify_base (priv->search_entry, GTK_STATE_NORMAL, no_result ? &color : NULL);
+#endif
+      gtk_widget_modify_bg (priv->search_entry, GTK_STATE_NORMAL, no_result ? &color : NULL);
+      gtk_widget_modify_bg (priv->search_entry, GTK_STATE_SELECTED, no_result ? &color : NULL);
+    }
+  if (i == 0)
+    {
+#ifdef MORE_FUNKY_COLOR_ON_SEARCH_ENTRY
+      gtk_widget_modify_base (priv->search_entry, GTK_STATE_NORMAL, &color);
+#endif
+      gtk_widget_modify_bg (priv->search_entry, GTK_STATE_NORMAL, &color);
+      gtk_widget_modify_bg (priv->search_entry, GTK_STATE_SELECTED, &color);
+    }
+  else if (no_result)
+    {
+#ifdef MORE_FUNKY_COLOR_ON_SEARCH_ENTRY
+      gtk_widget_modify_base (priv->search_entry, GTK_STATE_NORMAL, NULL);
+#endif
+      gtk_widget_modify_bg (priv->search_entry, GTK_STATE_NORMAL, NULL);
+      gtk_widget_modify_bg (priv->search_entry, GTK_STATE_SELECTED, NULL);
+    }
+}
+
 
 
 void
@@ -443,3 +544,121 @@ cb_key_pressed (XfmpcDbbrowser *dbbrowser,
   return TRUE;
 }
 
+static gboolean
+cb_button_pressed (XfmpcDbbrowser *dbbrowser,
+                   GdkEventButton *event)
+{
+  XfmpcDbbrowserPrivate    *priv = XFMPC_DBBROWSER_GET_PRIVATE (dbbrowser);
+  GtkTreeSelection         *selection;
+  GtkTreePath              *path;
+  
+  if (event->type != GDK_BUTTON_PRESS || event->button != 3)
+    return FALSE;
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
+  if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (priv->treeview),
+                                     event->x, event->y,
+                                     &path, NULL, NULL, NULL))
+    {
+      if (!gtk_tree_selection_path_is_selected (selection, path))
+        {
+          gtk_tree_selection_unselect_all (selection);
+          gtk_tree_selection_select_path (selection, path);
+        }
+      gtk_tree_path_free (path);
+    }
+
+  popup_menu (dbbrowser);
+
+  return TRUE;
+}
+
+static gboolean
+cb_popup_menu (XfmpcDbbrowser *dbbrowser)
+{
+  popup_menu (dbbrowser);
+
+  return TRUE;
+}
+
+static void
+popup_menu (XfmpcDbbrowser *dbbrowser)
+{
+  XfmpcDbbrowserPrivate    *priv = XFMPC_DBBROWSER_GET_PRIVATE (dbbrowser);
+
+  gtk_menu_popup (GTK_MENU (priv->menu),
+                  NULL, NULL,
+                  NULL, NULL,
+                  0,
+                  gtk_get_current_event_time ());
+}
+
+
+
+static void
+cb_search_entry_activated (XfmpcDbbrowser *dbbrowser)
+{
+  XfmpcDbbrowserPrivate    *priv = XFMPC_DBBROWSER_GET_PRIVATE (dbbrowser);
+  const gchar              *entry_text = gtk_entry_get_text (GTK_ENTRY (priv->search_entry));
+
+  if (entry_text[0] == '\0')
+    {
+      priv->is_searching = FALSE;
+      xfmpc_dbbrowser_reload (dbbrowser);
+
+      /* revert possible previous applied color */
+#ifdef MORE_FUNKY_COLOR_ON_SEARCH_ENTRY
+      gtk_widget_modify_base (priv->search_entry, GTK_STATE_NORMAL, NULL);
+#endif
+      gtk_widget_modify_bg (priv->search_entry, GTK_STATE_NORMAL, NULL);
+      gtk_widget_modify_bg (priv->search_entry, GTK_STATE_SELECTED, NULL);
+      return;
+    }
+
+  xfmpc_dbbrowser_search (dbbrowser, entry_text);
+}
+
+static gboolean
+cb_search_entry_key_released (XfmpcDbbrowser *dbbrowser,
+                              GdkEventKey *event)
+{
+  XfmpcDbbrowserPrivate *priv = XFMPC_DBBROWSER_GET_PRIVATE (dbbrowser);
+
+  if (event->type != GDK_KEY_RELEASE)
+    return FALSE;
+
+  if (event->keyval == GDK_Escape)
+    {
+      gtk_entry_set_text (GTK_ENTRY (priv->search_entry), "");
+    }
+
+  return TRUE;
+}
+
+static void
+cb_search_entry_changed (XfmpcDbbrowser *dbbrowser)
+{
+  XfmpcDbbrowserPrivate *priv = XFMPC_DBBROWSER_GET_PRIVATE (dbbrowser);
+
+  if (priv->search_timeout > 0)
+    g_source_remove (priv->search_timeout);
+
+  priv->search_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT, 642,
+                                             (GSourceFunc)timeout_search, dbbrowser,
+                                             (GDestroyNotify)timeout_search_destroy);
+}
+
+static gboolean
+timeout_search (XfmpcDbbrowser *dbbrowser)
+{
+  cb_search_entry_activated (dbbrowser);
+  return FALSE;
+}
+
+static void
+timeout_search_destroy (XfmpcDbbrowser *dbbrowser)
+{
+  XfmpcDbbrowserPrivate *priv = XFMPC_DBBROWSER_GET_PRIVATE (dbbrowser);
+  priv->search_timeout = 0;
+}
+
