@@ -41,7 +41,10 @@ enum
   SIG_TIME_CHANGED,
   SIG_VOLUME_CHANGED,
   SIG_STOPPED,
+  SIG_DATABASE_CHANGED,
   SIG_PLAYLIST_CHANGED,
+  SIG_REPEAT,
+  SIG_RANDOM,
   LAST_SIGNAL
 };
 
@@ -58,7 +61,7 @@ static void             xfmpc_mpdclient_initenv                 (XfmpcMpdclient
 static void             cb_xfmpc_mpdclient_status_changed       (MpdObj *mi,
                                                                  ChangedStatusType what,
                                                                  gpointer user_data);
-
+static gchar *          _get_formatted_name                     (mpd_Song *song);
 
 
 struct _XfmpcMpdclientClass
@@ -71,7 +74,10 @@ struct _XfmpcMpdclientClass
   void (*time_changed)      (XfmpcMpdclient *mpdclient, gint time, gint total_time, gpointer user_data);
   void (*volume_changed)    (XfmpcMpdclient *mpdclient, gint volume, gpointer user_data);
   void (*stopped)           (XfmpcMpdclient *mpdclient, gpointer user_data);
+  void (*database_changed)  (XfmpcMpdclient *mpdclient, gpointer user_data);
   void (*playlist_changed)  (XfmpcMpdclient *mpdclient, gpointer user_data);
+  void (*repeat)            (XfmpcMpdclient *mpdclient, gboolean repeat, gpointer user_data);
+  void (*random)            (XfmpcMpdclient *mpdclient, gboolean random, gpointer user_data);
 };
 
 struct _XfmpcMpdclient
@@ -186,6 +192,14 @@ xfmpc_mpdclient_class_init (XfmpcMpdclientClass *klass)
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
+  xfmpc_mpdclient_signals[SIG_DATABASE_CHANGED] =
+    g_signal_new ("database-changed", G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION,
+                  G_STRUCT_OFFSET (XfmpcMpdclientClass, database_changed),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
   xfmpc_mpdclient_signals[SIG_PLAYLIST_CHANGED] =
     g_signal_new ("playlist-changed", G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION,
@@ -193,6 +207,24 @@ xfmpc_mpdclient_class_init (XfmpcMpdclientClass *klass)
                   NULL, NULL,
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
+
+  xfmpc_mpdclient_signals[SIG_REPEAT] =
+    g_signal_new ("repeat", G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION,
+                  G_STRUCT_OFFSET (XfmpcMpdclientClass, repeat),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__BOOLEAN,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_BOOLEAN);
+
+  xfmpc_mpdclient_signals[SIG_RANDOM] =
+    g_signal_new ("random", G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION,
+                  G_STRUCT_OFFSET (XfmpcMpdclientClass, random),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__BOOLEAN,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_BOOLEAN);
 }
 
 static void
@@ -368,6 +400,18 @@ xfmpc_mpdclient_set_id (XfmpcMpdclient *mpdclient,
   return mpd_player_play_id (priv->mi, id) == MPD_OK ? TRUE : FALSE;
 }
 
+gboolean
+xfmpc_mpdclient_set_song_time (XfmpcMpdclient *mpdclient,
+                               guint time)
+{
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+
+  if (mpd_player_seek (priv->mi, time) != MPD_OK)
+    return FALSE;
+  else
+    return TRUE;
+}
+
 gboolean
 xfmpc_mpdclient_set_volume (XfmpcMpdclient *mpdclient,
                             guint8 volume)
@@ -381,12 +425,24 @@ xfmpc_mpdclient_set_volume (XfmpcMpdclient *mpdclient,
 }
 
 gboolean
-xfmpc_mpdclient_set_song_time (XfmpcMpdclient *mpdclient,
-                               guint time)
+xfmpc_mpdclient_set_repeat (XfmpcMpdclient *mpdclient,
+                            gboolean repeat)
 {
   XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
 
-  if (mpd_player_seek (priv->mi, time) != MPD_OK)
+  if (MPD_OK != mpd_player_set_repeat (priv->mi, repeat))
+    return FALSE;
+  else
+    return TRUE;
+}
+
+gboolean
+xfmpc_mpdclient_set_random (XfmpcMpdclient *mpdclient,
+                            gboolean random)
+{
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+
+  if (MPD_OK != mpd_player_set_random (priv->mi, random))
     return FALSE;
   else
     return TRUE;
@@ -499,6 +555,21 @@ xfmpc_mpdclient_get_volume (XfmpcMpdclient *mpdclient)
   return (volume < 0) ? 100 : volume;
 }
 
+gboolean
+xfmpc_mpdclient_get_repeat (XfmpcMpdclient *mpdclient)
+{
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+
+  return mpd_player_get_repeat (priv->mi);
+}
+
+gboolean
+xfmpc_mpdclient_get_random (XfmpcMpdclient *mpdclient)
+{
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+
+  return mpd_player_get_random (priv->mi);
+}
 gboolean
 xfmpc_mpdclient_is_playing (XfmpcMpdclient *mpdclient)
 {
@@ -533,6 +604,9 @@ cb_xfmpc_mpdclient_status_changed (MpdObj *mi,
   XfmpcMpdclient *mpdclient = XFMPC_MPDCLIENT (user_data);
   g_return_if_fail (G_LIKELY (NULL != user_data));
 
+  if (what & MPD_CST_DATABASE)
+    g_signal_emit_by_name (mpdclient, "database-changed");
+
   if (what & MPD_CST_PLAYLIST)
     g_signal_emit_by_name (mpdclient, "playlist-changed");
 
@@ -557,6 +631,14 @@ cb_xfmpc_mpdclient_status_changed (MpdObj *mi,
     g_signal_emit_by_name (mpdclient, "time-changed",
                            xfmpc_mpdclient_get_time (mpdclient),
                            xfmpc_mpdclient_get_total_time (mpdclient));
+
+  if (what & MPD_CST_REPEAT)
+    g_signal_emit_by_name (mpdclient, "repeat",
+                           xfmpc_mpdclient_get_repeat (mpdclient));
+
+  if (what & MPD_CST_RANDOM)
+    g_signal_emit_by_name (mpdclient, "random",
+                           xfmpc_mpdclient_get_random (mpdclient));
 }
 
 
@@ -614,16 +696,7 @@ xfmpc_mpdclient_playlist_read (XfmpcMpdclient *mpdclient,
 
   if (NULL != data)
     {
-      if (NULL != data->song->title)
-        {
-          if (data->song->artist)
-            *song = g_strdup_printf ("%s - %s", data->song->artist, data->song->title);
-          else
-            *song = g_strdup (data->song->title);
-        }
-      else
-        *song = g_path_get_basename (data->song->file);
-
+      *song = _get_formatted_name (data->song);
       *length = g_strdup_printf ("%d:%02d", data->song->time / 60, data->song->time % 60);
       *id = data->song->id;
     }
@@ -642,6 +715,17 @@ xfmpc_mpdclient_playlist_clear (XfmpcMpdclient *mpdclient)
   return TRUE;
 }
 
+gboolean
+xfmpc_mpdclient_database_refresh (XfmpcMpdclient *mpdclient)
+{
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+
+  if (mpd_database_update_dir (priv->mi, "/") != MPD_OK)
+    return FALSE;
+
+  return TRUE;
+}
+
 gboolean
 xfmpc_mpdclient_database_read (XfmpcMpdclient *mpdclient,
                                const gchar *dir,
@@ -649,8 +733,8 @@ xfmpc_mpdclient_database_read (XfmpcMpdclient *mpdclient,
                                gchar **basename,
                                gboolean *is_dir)
 {
-  static MpdData       *data = NULL;
   XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+  static MpdData        *data = NULL;
 
   if (NULL == data)
     data = mpd_database_get_directory (priv->mi, (gchar *)dir);
@@ -670,17 +754,7 @@ xfmpc_mpdclient_database_read (XfmpcMpdclient *mpdclient,
         case MPD_DATA_TYPE_SONG:
           *is_dir = FALSE;
           *filename = g_strdup (data->song->file);
-
-          if (NULL != data->song->title)
-            {
-              if (data->song->artist)
-                *basename = g_strdup_printf ("%s - %s", data->song->artist, data->song->title);
-              else
-                *basename = g_strdup (data->song->title);
-            }
-          else
-            *basename = g_path_get_basename (data->song->file);
-
+          *basename = _get_formatted_name (data->song);
           break;
 
         case MPD_DATA_TYPE_PLAYLIST:
@@ -701,13 +775,56 @@ xfmpc_mpdclient_database_read (XfmpcMpdclient *mpdclient,
 }
 
 gboolean
-xfmpc_mpdclient_database_refresh (XfmpcMpdclient *mpdclient)
+xfmpc_mpdclient_database_search (XfmpcMpdclient *mpdclient,
+                                 const gchar *query,
+                                 gchar **filename,
+                                 gchar **basename)
 {
   XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+  static MpdData        *data = NULL;
+  gchar                **queries;
+  gint                   i;
 
-  if (mpd_database_update_dir (priv->mi, "") != MPD_OK)
-    return FALSE;
+  if (NULL == data)
+    {
+      queries = g_strsplit (query, " ", -1);
 
-  return TRUE;
+      mpd_database_search_start (priv->mi, FALSE);
+      for (i = 0; queries[i] != NULL; i++)
+        mpd_database_search_add_constraint (priv->mi, MPD_TAG_ITEM_ANY, queries[i]);
+      data = mpd_database_search_commit (priv->mi);
+
+      g_strfreev (queries);
+    }
+  else
+    data = mpd_data_get_next (data);
+
+  if (NULL != data)
+    {
+      *filename = g_strdup (data->song->file);
+      *basename = _get_formatted_name (data->song);
+    }
+
+  return NULL != data;
+}
+
+
+
+static gchar *
+_get_formatted_name (mpd_Song *song)
+{
+  gchar *formatted_name;
+
+  if (NULL != song->title)
+    {
+      if (song->artist)
+        formatted_name = g_strdup_printf ("%s - %s", song->artist, song->title);
+      else
+        formatted_name = g_strdup (song->title);
+    }
+  else
+    formatted_name = g_path_get_basename (song->file);
+
+  return formatted_name;
 }
 
