@@ -59,10 +59,14 @@ gboolean                xfmpc_mpdclient_next                    (XfmpcMpdclient
 
 gboolean                xfmpc_mpdclient_set_id                  (XfmpcMpdclient *mpdclient,
                                                                  gint id);
-gboolean                xfmpc_mpdclient_set_volume              (XfmpcMpdclient *mpdclient,
-                                                                 guint8 volume);
 gboolean                xfmpc_mpdclient_set_song_time           (XfmpcMpdclient *mpdclient,
                                                                  guint time);
+gboolean                xfmpc_mpdclient_set_volume              (XfmpcMpdclient *mpdclient,
+                                                                 guint8 volume);
+gboolean                xfmpc_mpdclient_set_repeat              (XfmpcMpdclient *mpdclient,
+                                                                 gboolean repeat);
+gboolean                xfmpc_mpdclient_set_random              (XfmpcMpdclient *mpdclient,
+                                                                 gboolean random);
 gint                    xfmpc_mpdclient_get_pos                 (XfmpcMpdclient *mpdclient);
 
 gint                    xfmpc_mpdclient_get_id                  (XfmpcMpdclient *mpdclient);
@@ -81,6 +85,10 @@ gint                    xfmpc_mpdclient_get_total_time          (XfmpcMpdclient
 
 guint8                  xfmpc_mpdclient_get_volume              (XfmpcMpdclient *mpdclient);
 
+gboolean                xfmpc_mpdclient_get_repeat              (XfmpcMpdclient *mpdclient);
+
+gboolean                xfmpc_mpdclient_get_repeat              (XfmpcMpdclient *mpdclient);
+
 gboolean                xfmpc_mpdclient_is_playing              (XfmpcMpdclient *mpdclient);
 
 gboolean                xfmpc_mpdclient_is_stopped              (XfmpcMpdclient *mpdclient);
@@ -99,13 +107,17 @@ gboolean                xfmpc_mpdclient_playlist_read           (XfmpcMpdclient
                                                                  gchar **length);
 gboolean                xfmpc_mpdclient_playlist_clear          (XfmpcMpdclient *mpdclient);
 
+gboolean                xfmpc_mpdclient_database_refresh        (XfmpcMpdclient *mpdclient);
+
 gboolean                xfmpc_mpdclient_database_read           (XfmpcMpdclient *mpdclient,
                                                                  const gchar *dir,
                                                                  gchar **filename,
                                                                  gchar **basename,
                                                                  gboolean *is_dir);
-gboolean                xfmpc_mpdclient_database_refresh        (XfmpcMpdclient *mpdclient);
-
+gboolean                xfmpc_mpdclient_database_search         (XfmpcMpdclient *mpdclient,
+                                                                 const gchar *query,
+                                                                 gchar **filename,
+                                                                 gchar **basename);
 
 G_END_DECLS
 
